// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
syntax = "proto3";

import "basicmessages.proto";

package qtprotobufnamespace.tests;

message RepeatedStringMessage {
    repeated string testRepeatedString = 1;
}

message RepeatedDoubleMessage {
    repeated double testRepeatedDouble = 1;
}

message RepeatedBytesMessage {
    repeated bytes testRepeatedBytes = 1;
}

message RepeatedFloatMessage {
    repeated float testRepeatedFloat = 1;
}

message RepeatedComplexMessage {
    repeated ComplexMessage testRepeatedComplex = 1;
}

message RepeatedSIntMessage {
    repeated sint32 testRepeatedInt = 1;
}

message RepeatedIntMessage {
    repeated int32 testRepeatedInt = 1;
}

message RepeatedUIntMessage {
    repeated uint32 testRepeatedInt = 1;
}

message RepeatedSInt64Message {
    repeated sint64 testRepeatedInt = 1;
}

message RepeatedInt64Message {
    repeated int64 testRepeatedInt = 1;
}

message RepeatedUInt64Message {
    repeated uint64 testRepeatedInt = 1;
}

message RepeatedFixedIntMessage {
    repeated fixed32 testRepeatedInt = 1;
}

message RepeatedSFixedIntMessage {
    repeated sfixed32 testRepeatedInt = 1;
}

message RepeatedFixedInt64Message {
    repeated fixed64 testRepeatedInt = 1;
}

message RepeatedSFixedInt64Message {
    repeated sfixed64 testRepeatedInt = 1;
}

message RepeatedBoolMessage {
    repeated bool testRepeatedBool = 1;
}
