// Copyright (C) 2022 The Qt Company Ltd.
// Copyright (C) 2019 Alexey Edelev <semlanik@gmail.com>
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#ifndef QTGRPCGLOBAL_H
#define QTGRPCGLOBAL_H

#include <QtGrpc/qtgrpc-config.h>
#include <QtGrpc/qtgrpcexports.h>

#include <QtNetwork/qtnetwork-config.h>

#include <QtCore/qglobal.h>
#include <QtCore/qtconfigmacros.h>
#include <QtCore/qtcore-config.h>

#endif // QTGRPCGLOBAL_H
