// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTPROTOBUFGLOBAL_H
#define QTPROTOBUFGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtProtobuf/qtprotobufexports.h>

#endif
