/* This file is autogenerated. DO NOT CHANGE. All changes will be lost */

#ifndef _666_INVALID_IDENTIFIER_QPB_H
#define _666_INVALID_IDENTIFIER_QPB_H

#include "tst_qtgrpcgen_protobuf_invalid-identifier_exports.qpb.h"

#include <QtProtobuf/qprotobuflazymessagepointer.h>
#include <QtProtobuf/qprotobufmessage.h>
#include <QtProtobuf/qprotobufobject.h>
#include <QtProtobuf/qtprotobuftypes.h>

#include <QtCore/qbytearray.h>
#include <QtCore/qlist.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qshareddata.h>
#include <QtCore/qstring.h>

class Empty;
namespace Empty_QtProtobufNested {
enum class QtProtobufFieldEnum;
} // namespace Empty_QtProtobufNested

#ifdef QT_USE_PROTOBUF_LIST_ALIASES
using EmptyRepeated = QList<Empty>;
#endif // QT_USE_PROTOBUF_LIST_ALIASES

class Empty_QtProtobufData;
class Empty : public QProtobufMessage
{
    Q_PROTOBUF_OBJECT_EXPORT(QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT)

public:
    using QtProtobufFieldEnum = Empty_QtProtobufNested::QtProtobufFieldEnum;
    QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT Empty();
    QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT ~Empty();
    QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT Empty(const Empty &other);
    QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT Empty &operator =(const Empty &other);
    QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT Empty(Empty &&other) noexcept;
    Empty &operator =(Empty &&other) noexcept
    {
        swap(other);
        return *this;
    }
    void swap(Empty &other) noexcept
    {
        QProtobufMessage::swap(other);
        dptr.swap(other.dptr);
    }
    QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT Q_IMPLICIT operator QVariant() const;
    QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT static void registerTypes();

private:
    friend QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT bool comparesEqual(const Empty &lhs, const Empty &rhs) noexcept;
    friend bool operator==(const Empty &lhs, const Empty &rhs) noexcept
    {
        return comparesEqual(lhs, rhs);
    }
    friend bool operator!=(const Empty &lhs, const Empty &rhs) noexcept
    {
        return !comparesEqual(lhs, rhs);
    }
    QExplicitlySharedDataPointer<Empty_QtProtobufData> dptr;
};
namespace Empty_QtProtobufNested {
Q_NAMESPACE_EXPORT(QPB_TST_QTGRPCGEN_PROTOBUF_INVALID_IDENTIFIER_EXPORT)

} // namespace Empty_QtProtobufNested

#endif // _666_INVALID_IDENTIFIER_QPB_H
