/* This file is autogenerated. DO NOT CHANGE. All changes will be lost */

#ifndef _666_INVALID_IDENTIFIER_CLIENT_GRPC_QPB_H
#define _666_INVALID_IDENTIFIER_CLIENT_GRPC_QPB_H

#include "666-invalid-identifier.qpb.h"
#include "tst_qtgrpcgen_invalid-identifier_exports.qpb.h"

#include <QtGrpc/qgrpccallreply.h>
#include <QtGrpc/qgrpcclientbase.h>
#include <QtGrpc/qgrpcstream.h>

#include <memory>

namespace InvalidService {

class QPB_TST_QTGRPCGEN_INVALID_IDENTIFIER_EXPORT Client : public QGrpcClientBase
{
    Q_OBJECT

public:
    explicit Client(QObject *parent = nullptr);
    ~Client() override;

    [[nodiscard]]
    std::unique_ptr<QGrpcCallReply> Get(const Empty &arg);
    [[nodiscard]]
    std::unique_ptr<QGrpcCallReply> Get(const Empty &arg, const QGrpcCallOptions &options);


};
} // namespace InvalidService

#endif // _666_INVALID_IDENTIFIER_CLIENT_GRPC_QPB_H
