// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
syntax = "proto3";

import "basicmessages.proto";

package qtgrpc.tests.nomessages;

service TestService {
    rpc testMethod(qtprotobufnamespace.tests.SimpleStringMessage) returns (qtprotobufnamespace.tests.SimpleStringMessage) {}
}
