// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#include <QtGrpc/private/qtgrpclogging_p.h>

QT_BEGIN_NAMESPACE

Q_LOGGING_CATEGORY(Grpc, "qt.grpc", QtWarningMsg)

QT_END_NAMESPACE
