// Copyright (C) 2023 The Qt Company Ltd.
// Copyright (C) 2019 Alexey Edelev <semlanik@gmail.com>
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTPROTOBUFQTGUITYPESGLOBAL_H
#define QTPROTOBUFQTGUITYPESGLOBAL_H

#include <QtProtobufQtGuiTypes/qtprotobufqtguitypesexports.h>
#include <QtGui/qtguiglobal.h>

#endif // QTPROTOBUFQTGUITYPESGLOBAL_H
