// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
syntax = "proto3";

import "nopackageexternal.proto";

message EmptyMessage {}

enum TestEnum {
    LOCAL_ENUM_VALUE0 = 0;
    LOCAL_ENUM_VALUE1 = 1;
    LOCAL_ENUM_VALUE2 = 2;
    LOCAL_ENUM_VALUE3 = 5;
}

message SimpleIntMessage {
    int32 testFieldInt = 1;
}

message NoPackageExternalMessage {
    SimpleIntMessageExt testField = 1;
}

message NoPackageMessage {
    SimpleIntMessage testField = 1;
}
