/* This file is autogenerated. DO NOT CHANGE. All changes will be lost */

#include "666-invalid-identifier_client.grpc.qpb.h"

namespace InvalidService {
using namespace Qt::StringLiterals;

Client::Client(QObject *parent)
    : QGrpcClientBase("InvalidService"_L1, parent)
{
}

Client::~Client() = default;

std::unique_ptr<QGrpcCallReply> Client::Get(const Empty &arg)
{
    return Get(arg, {});
}


std::unique_ptr<QGrpcCallReply> Client::Get(const Empty &arg, const QGrpcCallOptions &options)
{
    auto reply = call("Get"_L1, arg, options);
    if (auto *replyPtr = reply.get(); replyPtr != nullptr) {
        setOperationResponseMetaType(replyPtr, QMetaType::fromType<Empty>());
    }
    return reply;
}

} // namespace InvalidService

