// Copyright (C) 2023 The Qt Company Ltd.
// Copyright (C) 2019 Alexey Edelev <semlanik@gmail.com>
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTPROTOBUFQTCORETYPESGLOBAL_H
#define QTPROTOBUFQTCORETYPESGLOBAL_H

#include <QtProtobufQtCoreTypes/qtprotobufqtcoretypesexports.h>
#include <QtCore/qglobal.h>

#endif // QTPROTOBUFQTCORETYPESGLOBAL_H
