// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
syntax = "proto3";

message Empty {}

service InvalidService {
    rpc Get(Empty) returns (Empty) {}
}
