// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#include <QtGrpcQuick/private/qtgrpcquicklogging_p.h>

QT_BEGIN_NAMESPACE

Q_LOGGING_CATEGORY(GrpcQuick, "qt.grpc.quick", QtWarningMsg)

QT_END_NAMESPACE
