// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
syntax = "proto3";

package qtprotobufnamespace.qttypes.tests;
import "QtGui/QtGui.proto";

message NestedQColorMessage {
    message QColorMessage {
        QtGui.QColor testField = 1;
    }
    NestedQColorMessage.QColorMessage testField = 1;
}
