/* This file is autogenerated. DO NOT CHANGE. All changes will be lost */


#include "qtprotobufgenminimal.qpb.h"

#include <QtProtobuf/qprotobufregistration.h>

#include <cmath>

namespace qt::protobuf {

class Empty_QtProtobufData : public QSharedData
{
public:
    Empty_QtProtobufData()
        : QSharedData()
    {
    }

    Empty_QtProtobufData(const Empty_QtProtobufData &other)
        : QSharedData(other)
    {
    }

};

Empty::~Empty() = default;

static constexpr struct {
    QtProtobufPrivate::QProtobufPropertyOrdering::Data data;
    const std::array<uint, 1> qt_protobuf_Empty_uint_data;
    const char qt_protobuf_Empty_char_data[19];
} qt_protobuf_Empty_metadata {
    // data
    {
        0, /* = version */
        0, /* = num fields */
        1, /* = field number offset */
        1, /* = property index offset */
        1, /* = field flags offset */
        17, /* = message full name length */
    },
    // uint_data
    {
        // JSON name offsets:
        18, /* = end-of-string-marker */
        // Field numbers:
        // Property indices:
        // Field flags:
    },
    // char_data
    /* metadata char_data: */
    "qt.protobuf.Empty\0" /* = full message name */
    /* field char_data: */
    ""
};

const QtProtobufPrivate::QProtobufPropertyOrdering Empty::staticPropertyOrdering = {
    &qt_protobuf_Empty_metadata.data
};

void Empty::registerTypes()
{
    qRegisterMetaType<Empty>();
    qRegisterMetaType<QList<Empty>>();
}

Empty::Empty()
    : QProtobufMessage(&Empty::staticMetaObject, &Empty::staticPropertyOrdering),
      dptr(new Empty_QtProtobufData)
{
}

Empty::Empty(const Empty &other)
    = default;
Empty &Empty::operator =(const Empty &other)
{
    Empty temp(other);
    swap(temp);
    return *this;
}
Empty::Empty(Empty &&other) noexcept
    = default;
Empty::operator QVariant() const
{
    return QVariant::fromValue(*this);
}
bool comparesEqual(const Empty &lhs, const Empty &rhs) noexcept
{
    return operator ==(static_cast<const QProtobufMessage&>(lhs),
                       static_cast<const QProtobufMessage&>(rhs));
}

} // namespace qt::protobuf

#include "moc_qtprotobufgenminimal.qpb.cpp"
