// Copyright (C) 2019 Alexey Edelev <semlanik@gmail.com>
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#include <QtProtobuf/private/qtprotobuflogging_p.h>

QT_BEGIN_NAMESPACE

Q_LOGGING_CATEGORY(Protobuf, "qt.protobuf", QtWarningMsg)

QT_END_NAMESPACE
